/*
Copyright (c) 2012 Kenrick Mock / Jeff Amato / Julian Bertmaring
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, 
this list of conditions and the following disclaimer.
 
Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.
 
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
 * GeoPixelFrame.java
 *
 * Created on Dec 31, 2011, 12:07:08 AM
 */
package geopixelcountjava5;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.Toolkit;
import java.awt.Cursor;
import java.io.*;
import javax.swing.JLabel;
//import javax.swing.filechooser.FileFilter;
//import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.JFileChooser;
import java.net.URL;
import javax.swing.JTable;
import java.awt.print.*;
/**
 *
 * @author Kenrick
 */
public class GeoPixelFrame extends javax.swing.JFrame {

   // Variable Declaration Block    
   private Image image = null; //Toolkit.getDefaultToolkit().getImage("");
   //create a buffered image but keep it empty for now -- this is the previous working image
   private BufferedImage imageTemp = null;
   // original image for toggle
   private BufferedImage imageToggle = null;
   //keep track if you are in grain replacement mode or not
   private boolean toggled = false;
   //boolean to keep track whether a picture is loaded or not
   private boolean pictureLoaded = false;
   //create a default color with rgb (throughout the process keeps track of the rgb of the color)
   private int rgbColor = java.awt.Color.RED.getRGB();
   //keep track of how many pixels there are (used for percentage calculation)
   private long percentage = 0;
   private long overall = 0;
   //keep track of how many grain replacements got made (used for the table)
   private int row = 0;
   //array of suggested colors for replacements
   private Color[] suggestedColors = {Color.RED, Color.GREEN, Color.BLUE, Color.PINK, Color.YELLOW, Color.CYAN};
   //index of colors used for the suggested color array
   private int colorIndex = 0;
   //set the default color to red to start with (throughout the process keeps track of what color)
   private Color color = java.awt.Color.RED;
   //array to keep track of which pixels are processed already
   private int[][] processedPixels;
   //keep track of the number of pixels of each grain replacement (used for percentage calculation)
   private long[] percentages = new long[10];
   //the selected color
   private int selectedColor = -1;
   //the original image
   private BufferedImage origImage = null;
   // Last path of files opened and saved
   private String lastOpenedPath = "";
   private String lastSavedPath = "";
   // Total nunber of pixels
   private long totalPixels = 0;
   
    
    /** Creates new form GeoPixelFrame */
    public GeoPixelFrame() {
        initComponents();
        /*
        try
        {
            URL url = new URL("http://www.math.uaa.alaska.edu/~afkjm/GeoPixelCounter/GeoPixelCounter.png");
            image = Toolkit.getDefaultToolkit().getImage(url);
            jLabelImage.setIcon(new javax.swing.ImageIcon(image));
        }
        catch (java.net.MalformedURLException err)
        {
            err.printStackTrace();
        }         
        */
        this.setTitle("GeoPixelCounter v1.0");
    }

    private void ProcessPixels(int rgb)
    {
      try
      {
        this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        //creating a new copy of the image
        BufferedImage imageTempNew = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
        //drawing the image
        // Need to start from the previous image without the current working color
        // added to it -- save in imageTemp?
        Graphics gNew = imageTempNew.createGraphics();
        gNew.drawImage(imageTemp, 0, 0, null);
        //give out the rgb value in the cmd window
        //System.out.println(rgb);
        //creating progress bar
        //jProgressBar1.setMaximum(100);
        //calculating the slider bars value for tolerance calculation
        int slider = jSlider1.getValue();
        //System.out.println("Slider value is " + slider );
        int difference = 0;
        int counter = 0;

       Color c = new Color(rgb);
       int redSrc = c.getRed();
       int greenSrc = c.getGreen();
       int blueSrc = c.getBlue();

        //for loops for checking the whole image for pixels of the color within tolerance
        // for loop for x coordinates
        for (int xNew = imageTemp.getMinX(); xNew < imageTemp.getTileWidth(); xNew++) {
            //for loop for y coordinates
            for (int yNew = imageTemp.getMinY(); yNew < imageTemp.getTileHeight(); yNew++) {
                //calculating the difference of the pixels rgb and the rgb of the orginal rgb
                // absolute value of (new pixels value) - (original pixels value)
                Color targetPixel = new Color(imageTemp.getRGB(xNew, yNew));
                int redTarget = targetPixel.getRed();
                int greenTarget = targetPixel.getGreen();
                int blueTarget = targetPixel.getBlue();
                difference = (int) Math.sqrt(
                        Math.pow(redTarget - redSrc, 2) +
                        Math.pow(greenTarget - greenSrc, 2) +
                        Math.pow(blueTarget - blueSrc, 2)
                        );

                //difference = Math.abs(imageTemp.getRGB(xNew, yNew)) - Math.abs(rgb);
 //               dummy = dummy - 1;
                //update the progress bar
  //              jProgressBar1.setValue(((1 - dummy) / rgb) / 100);
   //             jProgressBar1.setStringPainted(true);
                //if the processed pixel wasnt already edited go on
                if ((processedPixels[xNew][yNew] == -1) ||
                    (processedPixels[xNew][yNew] == colorIndex))
                {
                    //if the rgb value is within tolerance
                    if (difference < slider) {
                        //increase the counter by 1, paint it with the new color
                        //and add it to the array of processed pixels
                        imageTempNew.setRGB(xNew, yNew, rgbColor);
                        counter++;
                        processedPixels[xNew][yNew] = this.colorIndex;
                    }
                    else if (processedPixels[xNew][yNew] == colorIndex)
                    {
                        // Was marked, but don't mark it anymore because it's outside the range
                        processedPixels[xNew][yNew] = -1;
                    }
                }
            }
        }
        //after we are done we adding the value of the number of pixels to our global variable
        percentage = counter;
        //the image gets overwritten with the new updated image and displayed
        image = imageTempNew;
        jLabelImage.setIcon(new javax.swing.ImageIcon(image));
        //leaving grain replacement mode
        toggled = false;
        //change the status text back to normal
        jLabelInfo.setText(" ");
    }
    finally
    {
        this.setCursor(java.awt.Cursor.getDefaultCursor());
    }
}

   /*
    * Method to draw the image
    */
//    public void paintComponent(Graphics g) {
//        g.drawImage(image, 0, 0, null);
//    }        
 /*
  * Creating of the File Filter Guidelines to ensure
    only Image Files are opened
 */
  private final String[] okFileExtensions = new String[] {"jpg", "png", "gif"};
  private final String[] okSaveExtensions = new String[] {"csv"};
 /*
  * Method to check if the File is acceptable as an Image File
  */
  public boolean accept(File file)
  {
    for (String extension : okFileExtensions)
    {
      if (file.getName().toLowerCase().endsWith(extension))
      {
        return true;
      }
    }
    return false;
  }    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel3 = new javax.swing.JPanel();
        jLabel11 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();
        jLabel9 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jButton2 = new javax.swing.JButton();
        jLabel4 = new javax.swing.JLabel();
        btnSelectGrain = new javax.swing.JButton();
        jLabel6 = new javax.swing.JLabel();
        jSlider1 = new javax.swing.JSlider();
        btnToggleImage = new javax.swing.JButton();
        jLabel10 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jTextField1 = new javax.swing.JTextField();
        btnAddToList = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTable2 = new javax.swing.JTable();
        jLabel12 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jButton3 = new javax.swing.JButton();
        jLabel5 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        jLabel13 = new javax.swing.JLabel();
        jLabelInfo = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jLabelImage = new javax.swing.JLabel();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenuItem1 = new javax.swing.JMenuItem();
        jMenuItem2 = new javax.swing.JMenuItem();

        FormListener formListener = new FormListener();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setName("GeoPixelCounterFrame"); // NOI18N

        jLabel11.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel11.setText("<HTML>1.<HTML>");

        jButton1.setText("<html>Load Image</html>");
        jButton1.setActionCommand("btnLoadImage");
        jButton1.setName("btnLoadImage"); // NOI18N
        jButton1.addActionListener(formListener);

        jLabel9.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel9.setText("<HTML>2.<HTML>");

        jLabel2.setText("<HTML>Optional - Choose grain replacement color (a color not on the image)<HTML>");

        jButton2.setText("Change Color");
        jButton2.setName("btnChangeColor"); // NOI18N
        jButton2.addActionListener(formListener);

        jLabel4.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel4.setText("<HTML>3.<HTML>");

        btnSelectGrain.setText("<HTML>Select grain</HTML>");
        btnSelectGrain.setEnabled(false);
        btnSelectGrain.addActionListener(formListener);

        jLabel6.setText("<HTML>Adjust color  sensitivity match with slider.<HTML>");

        jSlider1.setFont(new java.awt.Font("Tahoma", 0, 8)); // NOI18N
        jSlider1.setMajorTickSpacing(15);
        jSlider1.setMaximum(256);
        jSlider1.addMouseListener(formListener);

        btnToggleImage.setLabel("Hold to Toggle Original Image");
        btnToggleImage.addMouseListener(formListener);

        jLabel10.setFont(new java.awt.Font("Tahoma", 0, 12)); // NOI18N
        jLabel10.setText("<HTML>4.<HTML>");

        jLabel7.setText("Name Mineral, Add to list and repeat at Step 2.  ");

        jLabel8.setText("Name:");

        btnAddToList.setText("<HTML>Add to list</HTML>");
        btnAddToList.setEnabled(false);
        btnAddToList.addActionListener(formListener);

        jTable2.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null},
                {null, null, null, null, null}
            },
            new String [] {
                "Mineral", "Color", "# of Pixels", "% Selected", "% Total"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.Object.class, java.lang.Integer.class, java.lang.Double.class, java.lang.Double.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jTable2.getTableHeader().setReorderingAllowed(false);
        jScrollPane2.setViewportView(jTable2);

        jLabel12.setFont(new java.awt.Font("Tahoma", 0, 12));
        jLabel12.setText("<HTML>5.<HTML>");

        jLabel3.setText("<HTML>Save table data after all minerals added.<HTML>");

        jButton3.setText("<html>Save Table</html>");
        jButton3.setActionCommand("btnSaveData");
        jButton3.setName("btnSaveData"); // NOI18N
        jButton3.addActionListener(formListener);

        jLabel5.setOpaque(true);
        jLabel5.setBackground(java.awt.Color.RED);
        jLabel5.setBackground(new java.awt.Color(255, 51, 51));
        jLabel5.setForeground(new java.awt.Color(255, 51, 51));

        jLabel1.setFont(new java.awt.Font("Tahoma", 0, 10)); // NOI18N
        jLabel1.setText("More");

        jLabel13.setFont(new java.awt.Font("Tahoma", 0, 10)); // NOI18N
        jLabel13.setText("Less");

        org.jdesktop.layout.GroupLayout jPanel3Layout = new org.jdesktop.layout.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel11, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel9, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jPanel3Layout.createSequentialGroup()
                                .add(jButton2)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 148, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jPanel3Layout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(btnSelectGrain, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 109, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jPanel3Layout.createSequentialGroup()
                                .add(10, 10, 10)
                                .add(btnToggleImage, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 239, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jSlider1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 229, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jPanel3Layout.createSequentialGroup()
                                .add(10, 10, 10)
                                .add(jLabel1)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 161, Short.MAX_VALUE)
                                .add(jLabel13)
                                .add(94, 94, 94))))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 300, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jPanel3Layout.createSequentialGroup()
                                .add(jLabel8)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 105, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(btnAddToList, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 100, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(jLabel7)))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(jLabel12, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 22, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jButton3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 102, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel3Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel11, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jButton1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(26, 26, 26)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel9, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(jLabel5, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jButton2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .add(18, 18, 18)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel4, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btnSelectGrain, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jLabel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jSlider1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(1, 1, 1)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(jLabel13))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(btnToggleImage)
                .add(18, 18, 18)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                        .add(jLabel7)
                        .add(jLabel10, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(31, 31, 31)
                        .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                            .add(jLabel8)
                            .add(jTextField1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(btnAddToList, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                    .add(jPanel3Layout.createSequentialGroup()
                        .add(65, 65, 65)
                        .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 126, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .add(18, 18, 18)
                .add(jPanel3Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel12, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jButton3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(37, Short.MAX_VALUE))
        );

        jLabelInfo.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        jLabelInfo.setText("GeoPixelCounter version 1.0");

        jScrollPane1.setBorder(new javax.swing.border.SoftBevelBorder(javax.swing.border.BevelBorder.RAISED));

        jLabelImage.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        jLabelImage.setText("No Image Loaded");
        jLabelImage.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        jLabelImage.setAutoscrolls(true);
        jLabelImage.addMouseListener(formListener);
        jScrollPane1.setViewportView(jLabelImage);

        jMenu1.setText("File");

        jMenuItem1.setText("Print");
        jMenuItem1.addActionListener(formListener);
        jMenu1.add(jMenuItem1);

        jMenuItem2.setText("Exit");
        jMenuItem2.addActionListener(formListener);
        jMenu1.add(jMenuItem2);
        jMenuItem2.getAccessibleContext().setAccessibleName("Exit");

        jMenuBar1.add(jMenu1);

        setJMenuBar(jMenuBar1);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel3, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 598, Short.MAX_VALUE)
                    .add(jLabelInfo, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 598, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(jLabelInfo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .add(18, 18, 18)
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 565, Short.MAX_VALUE)))
                .addContainerGap())
        );

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-986)/2, (screenSize.height-692)/2, 986, 692);
    }

    // Code for dispatching events from components to event handlers.

    private class FormListener implements java.awt.event.ActionListener, java.awt.event.MouseListener {
        FormListener() {}
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            if (evt.getSource() == jButton1) {
                GeoPixelFrame.this.LoadImageClicked(evt);
            }
            else if (evt.getSource() == jButton2) {
                GeoPixelFrame.this.jButton2ActionPerformed(evt);
            }
            else if (evt.getSource() == btnSelectGrain) {
                GeoPixelFrame.this.actionSelectGrain(evt);
            }
            else if (evt.getSource() == btnAddToList) {
                GeoPixelFrame.this.btnAddToListActionPerformed(evt);
            }
            else if (evt.getSource() == jButton3) {
                GeoPixelFrame.this.btnSaveTableActionPerformed(evt);
            }
            else if (evt.getSource() == jMenuItem2) {
                GeoPixelFrame.this.jMenuItem2ActionPerformed(evt);
            }
            else if (evt.getSource() == jMenuItem1) {
                GeoPixelFrame.this.jMenuItem1ActionPerformed(evt);
            }
        }

        public void mouseClicked(java.awt.event.MouseEvent evt) {
            if (evt.getSource() == jLabelImage) {
                GeoPixelFrame.this.jLabelImageMouseClicked(evt);
            }
        }

        public void mouseEntered(java.awt.event.MouseEvent evt) {
        }

        public void mouseExited(java.awt.event.MouseEvent evt) {
        }

        public void mousePressed(java.awt.event.MouseEvent evt) {
            if (evt.getSource() == btnToggleImage) {
                GeoPixelFrame.this.btnToggleImageMousePressed(evt);
            }
        }

        public void mouseReleased(java.awt.event.MouseEvent evt) {
            if (evt.getSource() == jSlider1) {
                GeoPixelFrame.this.actionMouseReleasedSlider(evt);
            }
            else if (evt.getSource() == btnToggleImage) {
                GeoPixelFrame.this.btnToggleImageMouseReleased(evt);
            }
        }
    }// </editor-fold>//GEN-END:initComponents

    private void LoadImageClicked(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_LoadImageClicked
        jLabelInfo.setText("");
        //Opening of the Image File
        javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
        chooser.setDialogTitle("Select image file to load");
        chooser.setSelectedFile(new File(this.lastOpenedPath));
        if (chooser.showOpenDialog(this.jButton1) != javax.swing.JFileChooser.APPROVE_OPTION)
        {
            return;
        }
        String filePath = chooser.getSelectedFile().getAbsolutePath();
        this.lastOpenedPath = filePath;

        //Filtering out non-image Files
        if(!accept(chooser.getSelectedFile()))

            //If incorrect bring the user back to the selection screen and inform him of the error
            javax.swing.JOptionPane.showMessageDialog(null,"Not a supported file type.  The program can read .gif, .jpg, .png");
        else {
            /*
             * If correct use the image and display it in the jLabel4
             * Set the pictureLoaded variable to true and allow grain replacement mode
             * also create a buffered image from the loaded image
             */
            try
            {
                image = Toolkit.getDefaultToolkit().getImage(filePath);
                jLabelImage.setText("");
                jLabelImage.setIcon(new javax.swing.ImageIcon(filePath));
                pictureLoaded = true;
                imageTemp = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
                Graphics g = imageTemp.createGraphics();
                g.drawImage(image, 0, 0, null);
                processedPixels = new int[imageTemp.getTileWidth()][imageTemp.getTileHeight()];
                for (int x = 0; x < imageTemp.getTileWidth(); x++)
                    for (int y = 0; y  < imageTemp.getTileHeight(); y++) {
                        processedPixels[x][y] = -1;
                    }
                // Remember original image
                origImage = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
                Graphics gNew = origImage.createGraphics();
                gNew.drawImage(image, 0, 0, null);
                // Track how many pixels there are for total percentage
                totalPixels = imageTemp.getTileWidth() * imageTemp.getTileHeight();
                // Enable buttons
                this.btnSelectGrain.setEnabled(true);
                // Clear table
                for (int row = 0; row < jTable2.getRowCount(); row++)
                {
                    for (int col=0; col <=4; col++)
                        jTable2.setValueAt(null, row, col);
                }
                jTable2.revalidate();            
                // Reset other vars
                this.row = 0;          
            }
            catch (Exception e)
            {
                javax.swing.JOptionPane.showMessageDialog(null,"Error loading image (too large?) : " + e.getMessage());                
            }
       }        
    }//GEN-LAST:event_LoadImageClicked

    private void jButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton2ActionPerformed
     /*
      * creating a JColorChooser and display it for choosing colors
      */
        javax.swing.JColorChooser chooser = new javax.swing.JColorChooser();
        Color newColor = chooser.showDialog(chooser, null, jLabel5.getBackground());
        rgbColor = newColor.getRGB();
        color = newColor;
     /*
      * If a color is selected change the displayed color to the chosen color
      */
        if (newColor != null) {
            jLabel5.setBackground(newColor);

        }        
    }//GEN-LAST:event_jButton2ActionPerformed

    private void actionSelectGrain(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_actionSelectGrain
        if (pictureLoaded ==true) {
            //enter the grain replacement mode
            toggled = true;
            //display text to inform the user that the grain replacement mode is active
            //and what he should do next
            jLabelInfo.setText("<HTML>Put the crosshairs over the pixel of the grain and click to select.</HTML>");
            this.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        }
        else
        {
            jLabelInfo.setText("<HTML>Load an image first.</HTML>");
        }        
    }//GEN-LAST:event_actionSelectGrain

    private void actionMouseReleasedSlider(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_actionMouseReleasedSlider
        // The slider changed values so go and recolor the image
        //System.out.println("State changed, val = " + jSlider1.getValue());
        if ((pictureLoaded == true) && (this.selectedColor != -1))
            ProcessPixels(this.selectedColor);
    }//GEN-LAST:event_actionMouseReleasedSlider

    private void jLabelImageMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jLabelImageMouseClicked
        //if the grain replacement mode is active and a picture is loaded it does something
        if (toggled == true && pictureLoaded == true) {
           /*
            * variable declaration block
            */
            //x value of the mouseclick
            int x = evt.getX();
            //y value of the mouseclick
            int y = evt.getY();
            //position given out in the cmd window
            //System.out.println(x);
            //System.out.println(y);
            //getting the rgb value of the pixel in the clicked position
            int rgb = this.origImage.getRGB(x,y);
            //change the status text back to normal
            ProcessPixels(rgb);
            this.selectedColor = rgb;  // Remember what color we picked
            this.setCursor(java.awt.Cursor.getDefaultCursor());
            // Allow to save in the list
            this.btnAddToList.setEnabled(true);
        }
    }//GEN-LAST:event_jLabelImageMouseClicked

    private void btnToggleImageMousePressed(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_btnToggleImageMousePressed
        // Save current image
        this.imageToggle = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
        Graphics gToggle = imageToggle.createGraphics();
        gToggle.drawImage(image, 0, 0, null);
        // Copy original
        BufferedImage imageTempNew = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
        //drawing the image
        Graphics gNew = imageTempNew.createGraphics();
        gNew.drawImage(origImage, 0, 0, null);
        image = imageTempNew;
        jLabelImage.setIcon(new javax.swing.ImageIcon(image));
    }//GEN-LAST:event_btnToggleImageMousePressed

    private void btnToggleImageMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_btnToggleImageMouseReleased
        BufferedImage imageTempNew = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
        //drawing the image
        Graphics gNew = imageTempNew.createGraphics();
        gNew.drawImage(this.imageToggle, 0, 0, null);
        image = imageTempNew;
        jLabelImage.setIcon(new javax.swing.ImageIcon(image));
    }//GEN-LAST:event_btnToggleImageMouseReleased

    private void btnAddToListActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnAddToListActionPerformed
         //read in the text from the texbox of the mineral name
        String mineral = jTextField1.getText();
        if (mineral.length() == 0)
        {
            jLabelInfo.setText("Type in a mineral name first.");
            return;
        }
        if (percentage == 0)
        {
            jLabelInfo.setText("Select a grain on the image first.");
            return;
        }
        jLabelInfo.setText("");

        //calculate the total number of pixels
        overall += percentage;
        long numberOfPixels = percentage;
        //add the number of pixel to an array for later use
        percentages[row]=percentage;
        //add the information of the current processed pixels to the table
        jTable2.setValueAt(mineral, row, 0);
        jTable2.setValueAt(color, row, 1);
        jTable2.setValueAt(numberOfPixels, row, 2);
        java.text.DecimalFormat decRound = new java.text.DecimalFormat("#.#");
        double pTotal = Double.parseDouble(decRound.format((double) numberOfPixels*100 / totalPixels));
        jTable2.setValueAt(pTotal, row, 4);
        //for the percent field go through all the rows and update the information
        for (int i = 0; i <= row; i++) {
            //while going through the rows go and calculate the percentages of this field and add it
            double pRunning = Double.parseDouble(decRound.format((double) percentages[i]*100 / overall));
            jTable2.setValueAt(pRunning , i, 3);
        }
        //increase the row counter for the array after you are done
        row++;
        //redraw the jTable
        jTable2.revalidate();
        //switch to the next default color
        colorIndex++;
        //when you are out of default colors go back to the first one
        if (colorIndex > suggestedColors.length) {
            colorIndex = 0;
        }
        //change the displayed color value to the default color
        jLabel5.setBackground(suggestedColors[colorIndex]);
        //change all the color depending variables to reflect the new color
        rgbColor = suggestedColors[colorIndex].getRGB();
        color = suggestedColors[colorIndex];

        // Copy current image to imageTemp to retain confirmed changes
        imageTemp = new BufferedImage(image.getWidth(null), image.getHeight(null), BufferedImage.TYPE_INT_RGB);
        Graphics gNew = imageTemp.createGraphics();
        gNew.drawImage(image, 0, 0, null);
        // Clear textbox with mineral name
        jTextField1.setText("");
        // Disable so we have to select another grain next
        this.btnAddToList.setEnabled(false);
        //this.btnToggleImage.setEnabled(true);
    }//GEN-LAST:event_btnAddToListActionPerformed

    private void btnSaveTableActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnSaveTableActionPerformed
        javax.swing.JFileChooser chooser = new javax.swing.JFileChooser();
        chooser.setDialogTitle("Select location to save the table of percentages.");
//        FileFilter filter = new FileNameExtensionFilter("CSV Files", "csv");
//        chooser.addChoosableFileFilter(filter);
        chooser.setSelectedFile(new File(this.lastSavedPath + "\\PixelPercentTable.csv"));

        if (chooser.showSaveDialog(this.jButton3) == javax.swing.JFileChooser.APPROVE_OPTION)
        {
            String filePath = chooser.getSelectedFile().getAbsolutePath();
            this.lastSavedPath = filePath;

            PrintWriter outData = null;	// Initialize stream to empty
            try
            {
                outData = new PrintWriter(new FileOutputStream(filePath));
                outData.println("Mineral,Pixels,PercentSelected,PercentTotal");
                for (int row = 0; row < jTable2.getRowCount(); row++)
                {
                    if (jTable2.getValueAt(row, 0) != null)
                    {
                         outData.println(jTable2.getValueAt(row, 0) + "," +
                                        jTable2.getValueAt(row, 2) + "," +
                                        jTable2.getValueAt(row, 3) + "," +
                                        jTable2.getValueAt(row, 4)
                                       );
                    }
                }
            }
            catch (Exception e)
            {
                    System.out.println(e);	// Print error
                    System.exit(0);		// Exit if an error
            }
            outData.close();	// Close file when done
        }
    }//GEN-LAST:event_btnSaveTableActionPerformed

    // Exit menu option
    private void jMenuItem2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem2ActionPerformed
        System.exit(0);
    }//GEN-LAST:event_jMenuItem2ActionPerformed

    // Print menu option
    private void jMenuItem1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMenuItem1ActionPerformed
        PrinterJob job = PrinterJob.getPrinterJob();
        job.setPrintable(new PrintImage(image));
        boolean doPrint = job.printDialog();
        if (doPrint)
        {
            try
            {
                job.print();
            }
            catch (PrinterException e)
            {
                /* The job did not successfully complete */
                javax.swing.JOptionPane.showMessageDialog(null,"Image did not print: " + e.getMessage()); 
            }
        }
    }//GEN-LAST:event_jMenuItem1ActionPerformed

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(GeoPixelFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(GeoPixelFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(GeoPixelFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(GeoPixelFrame.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new GeoPixelFrame().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnAddToList;
    private javax.swing.JButton btnSelectGrain;
    private javax.swing.JButton btnToggleImage;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton2;
    private javax.swing.JButton jButton3;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JLabel jLabelImage;
    private javax.swing.JLabel jLabelInfo;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuItem jMenuItem1;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JSlider jSlider1;
    private javax.swing.JTable jTable2;
    private javax.swing.JTextField jTextField1;
    // End of variables declaration//GEN-END:variables
}
